@@ -1,7 +1,7 @@
 import numpy as np
 from .model import Model
 
-from .lineshapes import (gaussian, lorentzian, voigt, pvoigt, pearson7,
+from .lineshapes import (gaussian, lorentzian, voigt, pvoigt, moffat, pearson7,
                          step, rectangle, breit_wigner, logistic,
                          students_t, lognormal, damped_oscillator,
                          expgaussian, skewed_gaussian, donaich,
@@ -30,7 +30,13 @@ def index_of(arr, val):
 
 def fwhm_expr(model):
     "return constraint expression for fwhm"
-    return "%.7f*%ssigma" % (model.fwhm_factor, model.prefix)
+    fmt = "{factor:.7f}*{prefix:s}sigma"
+    return fmt.format(factor=model.fwhm_factor, prefix=model.prefix)
+
+def height_expr(model):
+    "return constraint expression for maximum peak height"
+    fmt = "{factor:.7f}*{prefix:s}amplitude/{prefix:s}sigma"
+    return fmt.format(factor=model.height_factor, prefix=model.prefix)
 
 def guess_from_peak(model, y, x, negative, ampscale=1.0, sigscale=1.0):
     "estimate amp, cen, sigma for a peak, create params"
@@ -95,6 +101,18 @@ class ConstantModel(Model):
         pars['%sc' % self.prefix].set(value=data.mean())
         return update_param_vals(pars, self.prefix, **kwargs)
 
+class ComplexConstantModel(Model):
+    __doc__ = "x -> re+1j*im" + COMMON_DOC
+    def __init__(self, *args, **kwargs):
+        def constant(x, re, im):
+            return re + 1j*im
+        super(ComplexConstantModel, self).__init__(constant, *args, **kwargs)
+
+    def guess(self, data, **kwargs):
+        pars = self.make_params()
+        pars['%sre' % self.prefix].set(value=data.real.mean())
+        pars['%sim' % self.prefix].set(value=data.imag.mean())
+        return update_param_vals(pars, self.prefix, **kwargs)
 
 class LinearModel(Model):
     __doc__ = linear.__doc__ + COMMON_DOC if linear.__doc__ else ""
@@ -152,10 +170,12 @@ class PolynomialModel(Model):
 class GaussianModel(Model):
     __doc__ = gaussian.__doc__ + COMMON_DOC if gaussian.__doc__ else ""
     fwhm_factor = 2.354820
+    height_factor = 1./np.sqrt(2*np.pi)
     def __init__(self, *args, **kwargs):
         super(GaussianModel, self).__init__(gaussian, *args, **kwargs)
         self.set_param_hint('sigma', min=0)
         self.set_param_hint('fwhm', expr=fwhm_expr(self))
+        self.set_param_hint('height', expr=height_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
         pars = guess_from_peak(self, data, x, negative)
@@ -165,10 +185,12 @@ class GaussianModel(Model):
 class LorentzianModel(Model):
     __doc__ = lorentzian.__doc__ + COMMON_DOC if lorentzian.__doc__ else ""
     fwhm_factor = 2.0
+    height_factor = 1./np.pi
     def __init__(self, *args, **kwargs):
         super(LorentzianModel, self).__init__(lorentzian, *args, **kwargs)
         self.set_param_hint('sigma', min=0)
         self.set_param_hint('fwhm', expr=fwhm_expr(self))
+        self.set_param_hint('height', expr=height_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
         pars = guess_from_peak(self, data, x, negative, ampscale=1.25)
@@ -178,11 +200,13 @@ class LorentzianModel(Model):
 class VoigtModel(Model):
     __doc__ = voigt.__doc__ + COMMON_DOC if voigt.__doc__ else ""
     fwhm_factor = 3.60131
+    height_factor = 1./np.sqrt(2*np.pi)
     def __init__(self, *args, **kwargs):
         super(VoigtModel, self).__init__(voigt, *args, **kwargs)
         self.set_param_hint('sigma', min=0)
         self.set_param_hint('gamma', expr='%ssigma' % self.prefix)
         self.set_param_hint('fwhm',  expr=fwhm_expr(self))
+        self.set_param_hint('height', expr=height_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
         pars = guess_from_peak(self, data, x, negative,
@@ -192,9 +216,12 @@ class VoigtModel(Model):
 
 class PseudoVoigtModel(Model):
     __doc__ = pvoigt.__doc__ + COMMON_DOC if pvoigt.__doc__ else ""
+    fwhm_factor = 2.0
     def __init__(self, *args, **kwargs):
         super(PseudoVoigtModel, self).__init__(pvoigt, *args, **kwargs)
+        self.set_param_hint('sigma', min=0)
         self.set_param_hint('fraction', value=0.5)
+        self.set_param_hint('fwhm',  expr=fwhm_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
         pars = guess_from_peak(self, data, x, negative, ampscale=1.25)
@@ -202,6 +229,19 @@ class PseudoVoigtModel(Model):
         return update_param_vals(pars, self.prefix, **kwargs)
 
 
+class MoffatModel(Model):
+    __doc__ = moffat.__doc__ + COMMON_DOC if moffat.__doc__ else ""
+    def __init__(self, *args, **kwargs):
+        super(MoffatModel, self).__init__(moffat, *args, **kwargs)
+        self.set_param_hint('sigma', min=0)
+        self.set_param_hint('beta')
+        self.set_param_hint('fwhm', expr="2*%ssigma*sqrt(2**(1.0/%sbeta)-1)" % (self.prefix, self.prefix))
+
+    def guess(self, data, x=None, negative=False, **kwargs):
+        pars = guess_from_peak(self, data, x, negative, ampscale=0.5, sigscale=1.)
+        return update_param_vals(pars, self.prefix, **kwargs)
+
+
 class Pearson7Model(Model):
     __doc__ = pearson7.__doc__ + COMMON_DOC if pearson7.__doc__ else ""
     def __init__(self, *args, **kwargs):
@@ -335,6 +375,9 @@ class RectangleModel(Model):
     __doc__ = rectangle.__doc__ + COMMON_DOC if rectangle.__doc__ else ""
     def __init__(self, *args, **kwargs):
         super(RectangleModel, self).__init__(rectangle, *args, **kwargs)
+
+        self.set_param_hint('center1')
+        self.set_param_hint('center2')
         self.set_param_hint('midpoint',
                             expr='(%scenter1+%scenter2)/2.0' % (self.prefix,
                                                                 self.prefix))
@@ -354,11 +397,23 @@ class RectangleModel(Model):
 class ExpressionModel(Model):
     """Model from User-supplied expression
 
-%s
-    """ % COMMON_DOC
+Parameters
+----------
+expr:    string of mathematical expression for model.
+independent_vars: list of strings to be set as variable names
+missing: None, 'drop', or 'raise'
+    None: Do not check for null or missing values.
+    'drop': Drop null or missing observations in data.
+        Use pandas.isnull if pandas is available; otherwise,
+        silently fall back to numpy.isnan.
+    'raise': Raise a (more helpful) exception when data contains null
+        or missing values.
+prefix: NOT supported for ExpressionModel
+"""
 
     idvar_missing  = "No independent variable found in\n %s"
     idvar_notfound = "Cannot find independent variables '%s' in\n %s"
+    no_prefix      = "ExpressionModel does not support `prefix` argument"
     def __init__(self, expr, independent_vars=None, init_script=None,
                  *args, **kwargs):
 
@@ -370,8 +425,8 @@ class ExpressionModel(Model):
             self.asteval.eval(init_script)
 
         # save expr as text, parse to ast, save for later use
-        self.expr = expr
-        self.astcode = self.asteval.parse(expr)
+        self.expr = expr.strip()
+        self.astcode = self.asteval.parse(self.expr)
 
         # find all symbol names found in expression
         sym_names = get_ast_names(self.astcode)
@@ -401,6 +456,8 @@ class ExpressionModel(Model):
             raise ValueError(self.idvar_notfound % (lost, self.expr))
 
         kwargs['independent_vars'] = independent_vars
+        if 'prefix' in kwargs:
+            raise Warning(self.no_prefix)
 
         def _eval(**kwargs):
             for name, val in kwargs.items():
